@@ -9,6 +9,7 @@ from andes import routines   # NOQA
 from andes import utils      # NOQA
 from andes import variables  # NOQA
 from andes import thirdparty  # NOQA
+from andes import interop    # NOQA
 
 from andes.main import config_logger, run, load, prepare  # NOQA
 from andes.system import System                    # NOQA
@@ -18,5 +19,5 @@ from andes.utils.paths import get_case, list_cases  # NOQA
 __author__ = 'Hantao Cui'
 
 __all__ = ['main', 'plot', 'system', 'cli',
-           'utils', 'core', 'models', 'io', 'routines', 'variables', 'thirdparty',
+           'utils', 'core', 'models', 'io', 'routines', 'variables', 'thirdparty', 'interop',
            '__version__']
