@@ -1,6 +1,7 @@
-from andes.core import NumParam, ConstService, Algeb, LagAntiWindup, LeadLag, State, AntiWindup
+from andes.core import (Algeb, AntiWindup, ConstService, LagAntiWindup,
+                        LeadLag, NumParam, State,)
 from andes.core.block import DeadBand1
-from andes.models.governor.tgbase import TGBaseData, TGBase
+from andes.models.governor.tgbase import TGBase, TGBaseData
 
 
 class TGOV1Data(TGBaseData):
@@ -57,6 +58,10 @@ class TGOV1DBData(TGOV1Data):
 
 
 class TGOV1Model(TGBase):
+    """
+    Implement TGOV1 model.
+    """
+
     def __init__(self, system, config):
         TGBase.__init__(self, system, config)
 
@@ -70,17 +75,17 @@ class TGOV1Model(TGBase):
                           e_str='pref0 * R - pref',
                           )
 
-        self.wd = Algeb(info='Generator under speed',
+        self.wd = Algeb(info='Generator speed deviation',
                         unit='p.u.',
                         tex_name=r'\omega_{dev}',
                         v_str='0',
-                        e_str='ue * (wref - omega) - wd',
+                        e_str='ue * (omega - wref) - wd',
                         )
-        self.pd = Algeb(info='Pref plus under speed times gain',
+        self.pd = Algeb(info='Pref plus speed deviation times gain',
                         unit='p.u.',
                         tex_name="P_d",
                         v_str='ue * tm0',
-                        e_str='ue*(wd + pref + paux) * gain - pd')
+                        e_str='ue*(- wd + pref + paux) * gain - pd')
 
         self.LAG = LagAntiWindup(u=self.pd,
                                  K=1,
@@ -92,7 +97,7 @@ class TGOV1Model(TGBase):
                           T1=self.T2,
                           T2=self.T3,
                           )
-        self.pout.e_str = 'ue * (LL_y + Dt * wd) - pout'
+        self.pout.e_str = 'ue * (LL_y - Dt * wd) - pout'
 
 
 class TGOV1NModel(TGOV1Model):
@@ -105,18 +110,35 @@ class TGOV1NModel(TGOV1Model):
         self.pref.v_str = 'tm0'
         self.pref.e_str = 'pref0 - pref'
 
-        self.pd.e_str = 'ue*(wd * gain + pref + paux) - pd'
+        self.pd.e_str = 'ue*(-wd * gain + pref + paux) - pd'
 
 
 class TGOV1DBModel(TGOV1Model):
+    """
+    Model TGOV1 with deadband.
+    """
+
     def __init__(self, system, config):
         TGOV1Model.__init__(self, system, config)
         self.DB = DeadBand1(u=self.wd, center=0.0, lower=self.dbL,
                             upper=self.dbU, tex_name='DB',
-                            info='deadband for under speed',
+                            info='deadband for speed deviation',
                             )
-        self.pd.e_str = 'ue * (DB_y + pref + paux) * gain - pd'
-        self.pout.e_str = '(LL_y + Dt * DB_y) - pout'
+        self.pd.e_str = 'ue * (-DB_y + pref + paux) * gain - pd'
+        self.pout.e_str = '(LL_y - Dt * DB_y) - pout'
+
+
+class TGOV1NDBModel(TGOV1DBModel):
+    """
+    Implementation of TGOV1NDB
+    """
+
+    def __init__(self, system, config):
+        TGOV1DBModel.__init__(self, system, config)
+        self.pref.v_str = 'tm0'
+        self.pref.e_str = 'pref0 - pref'
+
+        self.pd.e_str = 'ue*(DB_y * gain + pref + paux) - pd'
 
 
 class TGOV1ModelAlt(TGBase):
@@ -133,17 +155,17 @@ class TGOV1ModelAlt(TGBase):
                           v_str='tm0 * R',
                           e_str='pref0 * R - pref',
                           )
-        self.wd = Algeb(info='Generator under speed',
+        self.wd = Algeb(info='Generator speed deviation',
                         unit='p.u.',
                         tex_name=r'\omega_{dev}',
                         v_str='0',
-                        e_str='ue * (wref - omega) - wd',
+                        e_str='ue * (omega - wref) - wd',
                         )
-        self.pd = Algeb(info='Pref plus under speed times gain',
+        self.pd = Algeb(info='Pref plus speed deviation times gain',
                         unit='p.u.',
                         tex_name="P_d",
                         v_str='tm0',
-                        e_str='(wd + pref + paux) * gain - pd')
+                        e_str='(- wd + pref + paux) * gain - pd')
 
         self.LAG_y = State(info='State in lag transfer function',
                            tex_name=r"x'_{LAG}",
@@ -168,7 +190,7 @@ class TGOV1ModelAlt(TGBase):
                           e_str='T2 / T3 * (LAG_y - LL_x) + LL_x - LL_y',
                           )
 
-        self.pout.e_str = 'ue * (LL_y + Dt * wd) - pout'
+        self.pout.e_str = 'ue * (LL_y - Dt * wd) - pout'
 
 
 class TGOV1(TGOV1Data, TGOV1Model):
@@ -211,3 +233,13 @@ class TGOV1DB(TGOV1DBData, TGOV1DBModel):
     def __init__(self, system, config):
         TGOV1DBData.__init__(self)
         TGOV1DBModel.__init__(self, system, config)
+
+
+class TGOV1NDB(TGOV1DBData, TGOV1NDBModel):
+    """
+    TGOV1N turbine governor model with speed input deadband.
+    """
+
+    def __init__(self, system, config):
+        TGOV1DBData.__init__(self)
+        TGOV1NDBModel.__init__(self, system, config)
