@@ -1,4 +1,5 @@
 import logging
+import inspect
 from collections import OrderedDict
 
 import numpy as np
@@ -412,8 +413,8 @@ class GroupBase:
         else:
             out += group_header + f'Group <{self.class_name}>\n' + group_header
 
-        if self.__doc__:
-            out += str(self.__doc__) + '\n\n'
+        if self.__doc__ is not None:
+            out += inspect.cleandoc(self.__doc__) + '\n\n'
 
         if len(self.common_params):
             out += 'Common Parameters: ' + ', '.join(self.common_params)
@@ -494,7 +495,7 @@ class StaticGen(GroupBase):
     def __init__(self):
         super().__init__()
         self.common_params.extend(('Sn', 'Vn', 'p0', 'q0', 'ra', 'xs', 'subidx'))
-        self.common_vars.extend(('p', 'q', 'a', 'v'))
+        self.common_vars.extend(('q', 'a', 'v'))
 
         self.SynGen = BackRef()
 
@@ -542,12 +543,40 @@ class SynGen(GroupBase):
     def __init__(self):
         super().__init__()
         self.common_params.extend(('Sn', 'Vn', 'fn', 'bus', 'M', 'D', 'subidx'))
-        self.common_vars.extend(('omega', 'delta', 'tm', 'te', 'vf', 'XadIfd', 'vd', 'vq', 'Id', 'Iq',
-                                 'a', 'v'))
+        self.common_vars.extend(('omega', 'delta', ))
+        self.idx_island = []
+        self.uid_island = []
+        self.delta_addr = []
 
         self.TurbineGov = BackRef()
         self.Exciter = BackRef()
 
+    def store_idx_island(self, bus_idx):
+        """
+        Get ``idx`` of generators in the given islanded. Also store the
+        addresses of the ``delta`` variable in the largest island.
+
+        This function can only be called after initializing dynamic devices.
+
+        Parameters
+        ----------
+        bus_idx : list
+            A list of bus idx in the largest island
+        """
+
+        idx_gen = list(self.uid.keys())
+        bus_gen = self.get(src='bus', idx=idx_gen, attr='v')
+
+        def intersect(lst1, lst2):
+            return list(set(lst1) & set(lst2))
+
+        bus_gen_island = intersect(bus_idx, bus_gen)
+        self.idx_island = self.find_idx(keys='bus',
+                                        values=bus_gen_island)
+        self.uid_island = self.idx2uid(self.idx_island)
+
+        self.delta_addr = self.get('delta', self.idx_island, 'a').astype(int)
+
 
 class RenGen(GroupBase):
     """
@@ -719,6 +748,14 @@ class PhasorMeasurement(GroupBase):
         self.common_vars.extend(('am', 'vm'))
 
 
+class PLL(GroupBase):
+    """Phase-locked loop models."""
+
+    def __init__(self):
+        super().__init__()
+        self.common_vars.extend(('am',))
+
+
 class Motor(GroupBase):
     """Induction Motor group
     """
