@@ -246,7 +246,8 @@ class REPCA1Model(Model):
 
         self.buss = DataSelect(self.busr, self.bus, info='selected bus (bus or busr)')
 
-        self.busfreq = DeviceFinder(self.busf, link=self.buss, idx_name='bus')
+        self.busfreq = DeviceFinder(self.busf, link=self.buss, idx_name='bus',
+                                    default_model='BusFreq')
 
         # from Bus
         self.v = ExtAlgeb(model='Bus', src='v', indexer=self.buss, tex_name='V',
@@ -492,21 +493,22 @@ class REPCA1Model(Model):
 
 class REPCA1(REPCA1Data, REPCA1Model):
     """
-    REPCA1 plat control model.
+    REPCA1: renewable energy power plat control model.
 
-    <PSS/E parser notice>
+    The output of the model, ``Pext`` and ``Qext``,  are the increment signals
+    of active and reactive power for the electrical control model.
 
-    In PSS/E dyr file:
+    Notes for PSS/E DYR parser:
 
-    1. If ICONs M+1 and M+2 are set to 0 when using generator power,
-       an error will be thrown by the parser, saying
-       "<REPCA1> cannot retrieve <bus1> from <ACLine> using <line>:
-       KeyError('Group <ACLine> does not contain device with idx=False')".
-       Manual effort is required to run the converted file. In the
-       REPCA1 sheet, input the idx of a line that connects to the RenGen bus.
+    1. If ICONs M+1 and M+2 are set to 0 when using generator power, an error
+       will be thrown by the parser, saying "<REPCA1> cannot retrieve <bus1>
+       from <ACLine> using <line>: KeyError('Group <ACLine> does not contain
+       device with idx=False')". Manual effort is required to run the converted
+       file. In the REPCA1 sheet, provide the idx of a line that connects to the
+       RenGen bus.
 
-    2. PSS/E enters ICONs M+3 as a string in single quotes. The pair of single quotes
-       need to be removed, or the conversion will fail.
+    2. PSS/E enters ICONs M+3 as a string in single quotes. The pair of single
+       quotes need to be removed, or the conversion will fail.
     """
 
     def __init__(self, system, config):
