@@ -2,20 +2,19 @@
 ANDES module for time-domain simulation.
 """
 
-import sys
-import os
-import time
 import importlib
 import logging
+import os
+import sys
+import time
 from collections import OrderedDict
 
 from andes.routines.base import BaseRoutine
-from andes.routines.daeint import method_map, Trapezoid
-
-from andes.utils.misc import elapsed, is_notebook, is_interactive
+from andes.routines.daeint import Trapezoid, method_map
+from andes.routines.criteria import deltadelta
+from andes.shared import matrix, np, pd, spdiag, tqdm, tqdm_nb
+from andes.utils.misc import elapsed, is_interactive, is_notebook
 from andes.utils.tab import Tab
-from andes.shared import tqdm, np, pd
-from andes.shared import matrix, spdiag
 
 logger = logging.getLogger(__name__)
 
@@ -42,6 +41,9 @@ class TDS(BaseRoutine):
                                      ('refresh_event', 0),
                                      ('test_init', 1),
                                      ('check_conn', 1),
+                                     ('check_conn', 1),
+                                     ('criteria', 1),
+                                     ('ddelta_limit', 180),
                                      ('g_scale', 1),
                                      ('reset_tiny', 1),
                                      ('qrt', 0),
@@ -65,6 +67,8 @@ class TDS(BaseRoutine):
                               refresh_event='refresh events at each step',
                               test_init='test if initialization passes',
                               check_conn='re-check connectivity after event',
+                              criteria='use criteria to stop simulation if unstable',
+                              ddelta_limit='delta diff. limit to be considered unstable, in degree',
                               g_scale='scale algebraic residuals with time step size',
                               reset_tiny='reset tiny residuals to zero to avoid chattering',
                               qrt='quasi-real-time stepping',
@@ -88,6 +92,7 @@ class TDS(BaseRoutine):
                               refresh_event=(0, 1),
                               test_init=(0, 1),
                               check_conn=(0, 1),
+                              criteria=(0, 1),
                               g_scale='positive',
                               reset_tiny=(0, 1),
                               qrt=(0, 1),
@@ -117,7 +122,7 @@ class TDS(BaseRoutine):
         self.deltatmin = 0
         self.deltatmax = 0
         self.h = 0
-        self.next_pc = 0
+        self.last_pc = 0.0
         self.Teye = None
         self.qg = np.array([])
         self.tol_zero = self.config.tol / 1e6
@@ -140,6 +145,7 @@ class TDS(BaseRoutine):
         self.test_ok = None
         self.qrt_start = None
         self.headroom = 0.0
+        self.call_stats = list()
 
         # internal storage for iterations
         self.x0 = None
@@ -323,12 +329,16 @@ class TDS(BaseRoutine):
             logger.debug("Initialization only is requested and done")
             return self.initialized
 
-        self.pbar = tqdm(total=100, unit='%', ncols=80, ascii=True,
-                         file=sys.stdout, disable=self.config.no_tqdm)
+        if is_notebook():
+            self.pbar = tqdm_nb(total=100, unit='%', file=sys.stdout,
+                                disable=self.config.no_tqdm)
+        else:
+            self.pbar = tqdm(total=100, unit='%', ncols=80, ascii=True,
+                             file=sys.stdout, disable=self.config.no_tqdm)
 
         if resume:
-            perc = round((dae.t - config.t0) / (config.tf - config.t0) * 100, 0)
-            self.next_pc = perc + 1
+            perc = round((dae.t - config.t0) / (config.tf - config.t0) * 100, 2)
+            self.last_pc = perc
             self.pbar.update(perc)
 
         self.qrt_start = time.time()
@@ -347,10 +357,17 @@ class TDS(BaseRoutine):
             else:
                 step_status = self._csv_step()
 
+            # record number of iterations and success flag
+            if system.config.call_stats:
+                self.call_stats.append((system.dae.t.tolist(), self.niter, step_status))
+
             if step_status:
                 dae.store()
 
                 self.streaming_step()
+                if self.check_criteria() is False:
+                    self.err_msg = 'Violated stability criteria.'
+                    self.busted = True
 
                 # check if the next step is critical time
                 self.do_switch()
@@ -359,9 +376,12 @@ class TDS(BaseRoutine):
 
                 # show progress in percentage
                 perc = max(min((dae.t - config.t0) / (config.tf - config.t0) * 100, 100), 0)
-                if perc >= self.next_pc:
-                    self.pbar.update(1)
-                    self.next_pc += 1
+                perc = round(perc, 2)
+
+                perc_diff = perc - self.last_pc
+                if perc_diff >= 1:
+                    self.pbar.update(perc_diff)
+                    self.last_pc = self.last_pc + perc_diff
 
                 # quasi-real-time check and wait (except for the last step)
                 if config.qrt and self.h > 0:
@@ -384,9 +404,6 @@ class TDS(BaseRoutine):
                     self.busted = True
                     break
 
-        self.pbar.close()
-        delattr(self, 'pbar')  # removed `pbar` so that System object can be serialized
-
         if self.busted:
             logger.error(self.err_msg)
             logger.error("Simulation terminated at t=%.4f s.", system.dae.t)
@@ -394,10 +411,15 @@ class TDS(BaseRoutine):
         elif system.dae.t == self.config.tf:
             succeed = True   # success flag
             system.exit_code += 0
+            self.pbar.update(100 - self.last_pc)
         else:
             system.exit_code += 1
 
-        _, s1 = elapsed(t0)
+        self.pbar.close()
+        self.pbar = None  # removed `pbar` so that System object can be serialized
+
+        t1, s1 = elapsed(t0)
+        self.exec_time = t1 - t0
         logger.info('Simulation completed in %s.', s1)
 
         if config.qrt:
@@ -713,8 +735,8 @@ class TDS(BaseRoutine):
         Perform one round of evaluation for one iteration step.
         The following operations are performed in order:
 
-        - discrete flags updating through ``l_update_var``
         - variable service updating through ``s_update_var``
+        - discrete flags updating through ``l_update_var``
         - evaluation of the right-hand-side of ``f``
         - equation-dependent discrete flags updating through ``l_update_eq``
         - evaluation of the right-hand-side of ``g``
@@ -725,13 +747,12 @@ class TDS(BaseRoutine):
         system = self.system
         system.dae.clear_fg()
 
+        system.s_update_var(models=models)  # update VarService
         system.l_update_var(models=models,
                             niter=self.niter,
                             err=self.mis[-1],
                             )
 
-        system.s_update_var(models=models)  # update VarService
-
         # evalute the RHS of `f` and check the limiters (anti-windup)
         # 12/08/2020: Moved `l_update_eq` to before `g_update`
         #   because some algebraic variables depend on pegged states.
@@ -826,12 +847,16 @@ class TDS(BaseRoutine):
             Index of the equation into the `g` array. Diff. eqns. are not counted in.
         """
         y_idx = y_idx.tolist()
-        logger.debug('Max. algebraic mismatch associated with <%s> [y_idx=%d]',
-                     self.system.dae.y_name[y_idx], y_idx)
+        logger.debug('--> Iteration Number: niter = %d', self.niter)
+        logger.debug('Max. algebraic equation mismatch:')
+        logger.debug('  <%s> [y_idx=%d]', self.system.dae.y_name[y_idx], y_idx)
+        logger.debug('  Variable value = %.4f', self.system.dae.y[y_idx])
+        logger.debug('  Mismatch value = %.4f', self.system.dae.g[y_idx])
+
         assoc_vars = self.system.dae.gy[y_idx, :]
         vars_idx = np.where(np.ravel(matrix(assoc_vars)))[0]
 
-        logger.debug('')
+        logger.debug('Related variable values:')
         logger.debug(f'{"y_index":<10} {"Variable":<20} {"Derivative":<20}')
         for v in vars_idx:
             v = v.tolist()
@@ -854,10 +879,13 @@ class TDS(BaseRoutine):
         eqns_idx = np.where(np.ravel(matrix(assoc_eqns)))[0]
         vars_idx = np.where(np.ravel(matrix(assoc_vars)))[0]
 
-        logger.debug('Max. correction is for variable %s [%d]', self.system.dae.xy_name[xy_idx], xy_idx)
-        logger.debug('Associated equation rhs is %20g', self.system.dae.fg[xy_idx])
+        logger.debug('Max. correction=%.4f for variable %s [%d]', self.inc[xy_idx],
+                     self.system.dae.xy_name[xy_idx], xy_idx)
+        logger.debug('Associated equation RHS is %20g', self.system.dae.fg[xy_idx])
         logger.debug('')
 
+        logger.debug('Related Jacobian elements:')
+        logger.debug(f'{"y_index":<10} {"Variable":<20} {"Derivative":<20}')
         logger.debug(f'{"xy_index":<10} {"Equation (row)":<20} {"Derivative":<20} {"Eq. Mismatch":<20}')
         for eq in eqns_idx:
             eq = eq.tolist()
@@ -879,7 +907,7 @@ class TDS(BaseRoutine):
         self.deltatmin = 0
         self.deltatmax = 0
         self.h = 0
-        self.next_pc = 0.1
+        self.last_pc = 0.0
         self.Teye = None
         self.qg = np.array([])
 
@@ -947,3 +975,12 @@ class TDS(BaseRoutine):
             name = 'trapezoid'
 
         self.method = method_map[name]()
+
+    def check_criteria(self):
+        """
+        Check stability criteria.
+        """
+
+        res = deltadelta(self.system.dae.x[self.system.SynGen.delta_addr],
+                         self.config.ddelta_limit)
+        return res
