@@ -14,7 +14,6 @@ import math
 import os
 import psutil
 
-from distutils.spawn import find_executable
 
 import coloredlogs  # NOQA
 import numpy as np  # NOQA
@@ -29,13 +28,13 @@ printing.options['dformat'] = '%.1f'
 printing.options['width'] = -1
 
 from andes.utils.paths import get_dot_andes_path  # NOQA
-from andes.utils.texttable import Texttable  # NOQA
 
 # --- SYSTEM INFO ---
 NCPUS_PHYSICAL = psutil.cpu_count(logical=False)
 
 # --- MATH CONSTANTS ---
 deg2rad = math.pi/180
+rad2deg = 180 / math.pi
 pi2o3 = math.pi * 2 / 3
 sqrt3 = math.sqrt(3)
 isqrt3 = math.sqrt(1/3)
@@ -44,7 +43,8 @@ isqrt3 = math.sqrt(1/3)
 jac_names = ('fx', 'fy', 'gx', 'gy')
 jac_types = ('c', '')
 
-dilled_vars = ['f_args', 'g_args', 'j_args', 's_args',
+dilled_vars = ['f_args', 'g_args', 'j_args',
+               's_args', 'sns_args',
                'ia_args', 'ii_args', 'ij_args',
                'ijac', 'jjac', 'vjac', 'j_names',
                'init_seq']
@@ -56,6 +56,7 @@ for jname in jac_names:
 
 # --- lazy import packages ---
 tqdm = LazyImport('from tqdm import tqdm')
+tqdm_nb = LazyImport('from tqdm.notebook import tqdm')
 
 pd = LazyImport('import pandas')
 numba = LazyImport('import numba')
@@ -63,6 +64,8 @@ cupy = LazyImport('import cupy')
 mpl = LazyImport('import matplotlib')
 unittest = LazyImport('import unittest')
 yaml = LazyImport('import yaml')
+pandapower = LazyImport('import pandapower')
+pypowsybl = LazyImport('import pypowsybl')
 
 plt = LazyImport('from matplotlib import pyplot')
 Pool = LazyImport('from pathos.multiprocessing import Pool')
@@ -73,8 +76,12 @@ fsolve = LazyImport('from scipy.optimize import fsolve')
 solve_ivp = LazyImport('from scipy.integrate import solve_ivp')
 odeint = LazyImport('from scipy.integrate import odeint')
 
+Oct2PyError = LazyImport('from oct2py import Oct2PyError')
+
 
 # --- Shared functions ---
+find_executable = LazyImport('from distutils.spawn import find_executable')
+
 
 def set_latex():
     """
@@ -104,3 +111,22 @@ def set_latex():
         return True
 
     return False
+
+
+def set_font(family='serif', size=12, style='normal', weight='normal'):
+    """
+    Sets the font for matplotlib.
+
+    Parameters
+    ----------
+    family : str
+        Font family.
+    size : int
+        Font size.
+    style : str
+        Font style.
+    weight : str
+        Font weight.
+    """
+
+    mpl.rc('font', family=family, size=size, style=style, weight=weight)
