@@ -2,7 +2,7 @@
 System class for power system data and methods
 """
 
-#  [ANDES] (C)2015-2021 Hantao Cui
+#  [ANDES] (C)2015-2022 Hantao Cui
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@ import logging
 import os
 import sys
 import time
-from collections import OrderedDict
+from collections import OrderedDict, defaultdict
 from typing import Dict, Optional, Tuple, Union
 
 import dill
@@ -29,11 +29,11 @@ from andes.io.streaming import Streaming
 from andes.models import file_classes
 from andes.models.group import GroupBase
 from andes.routines import all_routines
-from andes.shared import (NCPUS_PHYSICAL, Pool, Process, dilled_vars, jac_names, matrix, np,
-                          sparse, spmatrix,)
+from andes.shared import (NCPUS_PHYSICAL, Pool, Process, dilled_vars,
+                          jac_names, matrix, np, sparse, spmatrix)
 from andes.utils.misc import elapsed
 from andes.utils.paths import (andes_root, confirm_overwrite, get_config_path,
-                               get_pkl_path, get_pycode_path,)
+                               get_pkl_path, get_pycode_path)
 from andes.utils.tab import Tab
 from andes.variables import DAE, FileMan
 
@@ -144,6 +144,7 @@ class System:
                                      ('numba_parallel', 0),
                                      ('numba_nopython', 0),
                                      ('yapf_pycode', 0),
+                                     ('call_stats', 0),
                                      ('np_divide', 'warn'),
                                      ('np_invalid', 'warn'),
                                      ('pickle_path', get_pkl_path())
@@ -160,6 +161,7 @@ class System:
                               numba_parallel='enable parallel for numba.jit',
                               numba_nopython='nopython mode for numba',
                               yapf_pycode='format generated code with yapf',
+                              call_stats='store statistics of function calls',
                               np_divide='treatment for division by zero',
                               np_invalid='treatment for invalid floating-point ops.',
                               pickle_path='path models should be (un)dilled to/from',
@@ -175,6 +177,7 @@ class System:
                               numba_parallel=(0, 1),
                               numba_nopython=(0, 1),
                               yapf_pycode=(0, 1),
+                              call_stats=(0, 1),
                               np_divide={'ignore', 'warn', 'raise', 'call', 'print', 'log'},
                               np_invalid={'ignore', 'warn', 'raise', 'call', 'print', 'log'},
                               )
@@ -198,6 +201,7 @@ class System:
         self._setters = dict(f=list(), g=list(), x=list(), y=list())
         self.antiwindups = list()
         self.no_check_init = list()  # states for which initialization check is omitted
+        self.call_stats = defaultdict(dict)  # call statistics storage
 
         # internal flags
         self.is_setup = False        # if system has been setup
@@ -371,11 +375,11 @@ class System:
         model_names = list(models.keys())
         model_list = list()
 
-        for file, cls_list in file_classes.items():
+        for fname, cls_list in file_classes:
             for model_name in cls_list:
                 if model_name not in model_names:
                     continue
-                the_module = importlib.import_module('andes.models.' + file)
+                the_module = importlib.import_module('andes.models.' + fname)
                 the_class = getattr(the_module, model_name)
                 model_list.append(the_class(system=None, config=self._config_object))
 
@@ -558,6 +562,8 @@ class System:
         """
         Set addresses for differential and algebraic variables.
         """
+
+        # --- Phase 1: set internal variable addresses ---
         for mdl in models.values():
             if mdl.flags.address is True:
                 logger.debug('%s internal address exists', mdl.class_name)
@@ -565,33 +571,32 @@ class System:
             if mdl.n == 0:
                 continue
 
-            # set internal variable addresses
             logger.debug('Setting internal address for %s', mdl.class_name)
-            n = mdl.n
-            m0 = self.dae.m
-            n0 = self.dae.n
-            m_end = m0 + len(mdl.algebs) * n
-            n_end = n0 + len(mdl.states) * n
+
             collate = mdl.flags.collate
+            ndevice = mdl.n
 
-            if not collate:
-                for idx, item in enumerate(mdl.algebs.values()):
-                    item.set_address(np.arange(m0 + idx * n, m0 + (idx + 1) * n), contiguous=True)
-                for idx, item in enumerate(mdl.states.values()):
-                    item.set_address(np.arange(n0 + idx * n, n0 + (idx + 1) * n), contiguous=True)
-            else:
-                for idx, item in enumerate(mdl.algebs.values()):
-                    item.set_address(np.arange(m0 + idx, m_end, len(mdl.algebs)), contiguous=False)
-                for idx, item in enumerate(mdl.states.values()):
-                    item.set_address(np.arange(n0 + idx, n_end, len(mdl.states)), contiguous=False)
+            # get and set internal variable addresses
+            xaddr = self.dae.request_address('x', ndevice=ndevice,
+                                             nvar=len(mdl.states),
+                                             collate=mdl.flags.collate,
+                                             )
+            yaddr = self.dae.request_address('y', ndevice=ndevice,
+                                             nvar=len(mdl.algebs),
+                                             collate=mdl.flags.collate,
+                                             )
 
-            self.dae.m = m_end
-            self.dae.n = n_end
+            for idx, item in enumerate(mdl.states.values()):
+                item.set_address(xaddr[idx], contiguous=not collate)
+            for idx, item in enumerate(mdl.algebs.values()):
+                item.set_address(yaddr[idx], contiguous=not collate)
 
-        # set external variable addresses
+        # --- Phase 2: set external variable addresses ---
+        # NOTE: this step will retrieve the number of variables (item.n) for
+        # Phase 3.
         for mdl in models.values():
             # handle external groups
-            for name, instance in mdl.cache.vars_ext.items():
+            for instance in mdl.cache.vars_ext.values():
                 ext_name = instance.model
                 try:
                     ext_model = self.__dict__[ext_name]
@@ -605,7 +610,7 @@ class System:
                                  mdl.class_name, instance.name, instance.model,
                                  instance.indexer.name, repr(e))
 
-        # set external variable RHS addresses
+        #  --- Phase 3: set external variable RHS addresses ---
         for mdl in models.values():
             if mdl.flags.address is True:
                 logger.debug('%s RHS address exists', mdl.class_name)
@@ -880,9 +885,12 @@ class System:
         """
         Perform per unit value conversion.
 
-        This function calculates the per unit conversion factors, stores input parameters to `vin`, and perform
-        the conversion.
+        This function calculates the per unit conversion factors, stores input
+        parameters to `vin`, and perform the conversion.
         """
+        # `Sb`, `Vb` and `Zb` are the system base, bus base values
+        # `Sn`, `Vn` and `Zn` are the device bases
+
         Sb = self.config.mva
 
         for mdl in self.models.values():
@@ -935,6 +943,10 @@ class System:
                 for p in mdl.find_param(prop).values():
                     p.set_pu_coeff(coeff)
 
+            # store coeffs and bases back in models.
+            mdl.coeffs = coeffs
+            mdl.bases = {'Sn': Sn, 'Sb': Sb, 'Vn': Vn, 'Vb': Vb, 'Zn': Zn, 'Zb': Zb}
+
     def l_update_var(self, models: OrderedDict, niter=None, err=None):
         """
         Update variable-based limiter discrete states by calling ``l_update_var`` of models.
@@ -1257,7 +1269,7 @@ class System:
                     self.Bus.msw_island.append(idx)
 
         # --- Post processing ---
-        # extend islanded buses, each in a list
+        # 1. extend islanded buses, each in a list
         if len(self.Bus.islanded_buses) > 0:
             self.Bus.islands.extend([[item] for item in self.Bus.islanded_buses])
 
@@ -1266,6 +1278,19 @@ class System:
         else:
             self.Bus.islands.extend(self.Bus.island_sets)
 
+        # 2. find generators in the largest island
+        if self.TDS.config.criteria and self.TDS.initialized:
+            lg_island = None
+            for item in self.Bus.islands:
+                if lg_island is None:
+                    lg_island = item
+                    continue
+                if len(item) > len(lg_island):
+                    lg_island = item
+
+            lg_bus_idx = [self.Bus.idx.v[ii] for ii in lg_island]
+            self.SynGen.store_idx_island(lg_bus_idx)
+
         if info is True:
             self.summary()
 
@@ -1601,10 +1626,13 @@ class System:
 
             # services
             for instance in model.services.values():
-                if instance.v_str is not None:
+                if (instance.v_str is not None) and instance.sequential is True:
                     sv_name = f'{instance.name}_svc'
                     model.calls.s[instance.name] = pycode_model.__dict__[sv_name]
 
+            # services - non sequential
+            model.calls.sns = pycode_model.__dict__.get("sns_update")
+
             # load initialization; assignment
             for instance in model.cache.all_vars.values():
                 if instance.v_str is not None:
@@ -1686,6 +1714,12 @@ class System:
         for name, mdl in models.items():
             ret[name] = getattr(mdl, method)(*args, **kwargs)
 
+            if self.config.call_stats:
+                if method not in self.call_stats[name]:
+                    self.call_stats[name][method] = 1
+                else:
+                    self.call_stats[name][method] += 1
+
         return ret
 
     def _check_group_common(self):
@@ -1800,9 +1834,9 @@ class System:
 
         ``system.models['Bus']`` points the same instance as ``system.Bus``.
         """
-        for file, cls_list in file_classes.items():
+        for fname, cls_list in file_classes:
             for model_name in cls_list:
-                the_module = importlib.import_module('andes.models.' + file)
+                the_module = importlib.import_module('andes.models.' + fname)
                 the_class = getattr(the_module, model_name)
                 self.__dict__[model_name] = the_class(system=self, config=self._config_object)
                 self.models[model_name] = self.__dict__[model_name]
@@ -1942,8 +1976,8 @@ class System:
 
     def _list2array(self):
         """
-        Helper function to call models' ``list2array`` method, which
-        usually performs memory preallocation.
+        Helper function to call models' ``list2array`` method, which usually
+        performs memory preallocation.
         """
         self.call_models('list2array', self.models)
 
@@ -1951,7 +1985,8 @@ class System:
         """
         Set configuration for the System object.
 
-        Config for models are routines are passed directly to their constructors.
+        Config for models are routines are passed directly to their
+        constructors.
         """
         if config is not None:
             # set config for system
@@ -1966,7 +2001,8 @@ class System:
         Returns
         -------
         dict
-            a dict containing the config from devices; class names are keys and configs in a dict are values.
+            a dict containing the config from devices; class names are keys and
+            configs in a dict are values.
         """
         config_dict = configparser.ConfigParser()
         config_dict[self.__class__.__name__] = self.config.as_dict()
@@ -1988,8 +2024,8 @@ class System:
         Parameters
         ----------
         conf_path : None or str
-            Path to the config file. If is `None`, the function body
-            will not run.
+            Path to the config file. If is `None`, the function body will not
+            run.
 
         Returns
         -------
@@ -2005,20 +2041,22 @@ class System:
 
     def save_config(self, file_path=None, overwrite=False):
         """
-        Save all system, model, and routine configurations to an rc-formatted file.
+        Save all system, model, and routine configurations to an rc-formatted
+        file.
 
         Parameters
         ----------
         file_path : str, optional
             path to the configuration file default to `~/andes/andes.rc`.
         overwrite : bool, optional
-            If file exists, True to overwrite without confirmation.
-            Otherwise prompt for confirmation.
+            If file exists, True to overwrite without confirmation. Otherwise
+            prompt for confirmation.
 
         Warnings
         --------
-        Saved config is loaded back and populated *at system instance creation time*.
-        Configs from the config file takes precedence over default config values.
+        Saved config is loaded back and populated *at system instance creation
+        time*. Configs from the config file takes precedence over default config
+        values.
         """
         if file_path is None:
             andes_path = os.path.join(os.path.expanduser('~'), '.andes')
@@ -2046,13 +2084,24 @@ class System:
             A table-formatted string for the groups and models
         """
 
+        def rst_ref(name, export):
+            """
+            Refer to the model in restructuredText mode so that
+            it renders as a hyperlink.
+            """
+
+            if export == 'rest':
+                return ":ref:`" + name + '`'
+            else:
+                return name
+
         pairs = list()
         for g in self.groups:
             models = list()
             for m in self.groups[g].models:
-                models.append(m)
+                models.append(rst_ref(m, export))
             if len(models) > 0:
-                pairs.append((g, ', '.join(models)))
+                pairs.append((rst_ref(g, export), ', '.join(models)))
 
         tab = Tab(title='Supported Groups and Models',
                   header=['Group', 'Models'],
@@ -2064,9 +2113,9 @@ class System:
 
     def as_dict(self, vin=False, skip_empty=True):
         """
-        Return system data as a dict where the keys are model names
-        and values are dicts. Each dict has parameter names as keys
-        and corresponding data in an array as values.
+        Return system data as a dict where the keys are model names and values
+        are dicts. Each dict has parameter names as keys and corresponding data
+        in an array as values.
 
         Returns
         -------
@@ -2086,8 +2135,8 @@ class System:
         """
         Fixes addressing issues after loading a snapshot.
 
-        This function properly sets ``v`` and ``e`` of internal
-        variables as views of the corresponding DAE arrays.
+        This function properly sets ``v`` and ``e`` of internal variables as
+        views of the corresponding DAE arrays.
 
         Inputs will be refreshed for each model.
         """
@@ -2203,3 +2252,21 @@ def _set_z_name(mdl, dae, dests):
                 dests[0].append(f'{name} {_append_model_name(mdl_name, idx_item)}')
                 dests[1].append(rf'${item.tex_name}$ {_append_model_name(mdl_name, idx_item)}')
                 dae.o += 1
+
+
+def example(setup=True, no_output=True, **kwargs):
+    """
+    Return an :py:class:`andes.system.System` object for the
+    ``ieee14_linetrip.xlsx`` as an example.
+
+    This function is useful when a user wants to quickly get a
+    System object for testing.
+
+    Returns
+    -------
+    System
+        An example :py:class:`andes.system.System` object.
+    """
+
+    return andes.load(andes.get_case("ieee14/ieee14_linetrip.xlsx"),
+                      setup=setup, no_output=no_output, **kwargs)
